# How to Be Fast and Not Furious: Looking Under the Hood of CPU Cache Prefetching
This repository holds the source code related to our paper [How to Be Fast and Not Furious: Looking Under the Hood of CPU Cache Prefetching](https://dl.acm.org/doi/pdf/10.1145/3662010.3663451).

## Build the framekwork

    cmake . -DCMAKE_BUILD_TYPE=Release -DCMAKE_C_COMPILER=clang-17 -DCMAKE_CXX_COMPILER=clang++-17
    make 

The benchmark will try to infer the native cache line size. However, if needed, you can force a **specific cache line size** by

    cmake . -DCACHE_LINE_SIZE=128 -DCMAKE_BUILD_TYPE=Release -DCMAKE_C_COMPILER=clang-17 -DCMAKE_CXX_COMPILER=clang++-17

## Install Python Dependencies

    python3 -m venv .venv
    source .venv/bin/activate
    python3 -m pip install -r script/requirements.txt

## Execute Benchmarks
    
    source .venv/bin/activate
    python3 script/execute.py --gb 4 --interleaved-prefetching --huge-pages --profile perf/intel-cascade-lake.json

The script will write all results into `out.csv`.
Please make sure that you appended a **perf profile** matching your underlying hardware (take a look into  `perf/`).


## Executing Benchmarks only for trends

    ./script/execute-trend.sh

This is a trimmed variant for only recording data needed for plotting the trend.

## Plot

    source .venv/bin/activate
    python3 script/paper.py --in plots/ --out /path/to/plot

Make sure to adapt the script `script/paper.py` and add your data.
    
## Experimental Environment
This is the configuration we used to record the data on our machines.

#### Size:
* ARM Consumer: `1GB` 
* Intel/AMD/ARM Server: `4GB`

#### Numa Balancing: OFF

    echo 0 > /proc/sys/kernel/numa_balancing

#### Performance Governor

    cpupower frequency-set -g performance

Activate Performance Counter

    sysctl -w kernel.perf_event_paranoid=-1

#### Needed Experiments:

     python3 script/execute.py --profile perf/intel-cascade-lake.json --gb 4 --interleaved --huge-pages