import argparse
import subprocess

parser = argparse.ArgumentParser()
parser.add_argument("--profile", type=str, help="Perf profile", required=True)
parser.add_argument("--sample", type=str, help="Event to sample", required=True)
parser.add_argument("--gb", type=int, help="Size of the global array to read.", default=4)
parser.add_argument("--l3", dest="is_l3_pref", help="If set, out files will contain an indicator that the L3 (or LLC) prefetcher is active.", action="store_true")
args = parser.parse_args()

block_size = 16

for is_l1_pref in [True, False]:
    for is_l2_pref in [True, False]:

        core_prefetcher_msr = 12 if not is_l1_pref else 0
        if not is_l2_pref:
            core_prefetcher_msr += 3

        subprocess.call(f'sudo wrmsr -a 0x1A4 {core_prefetcher_msr}', shell=True)

        name_prefetch_hint = ''
        if is_l1_pref:
            name_prefetch_hint += '-l1'
        if is_l2_pref:
            name_prefetch_hint += '-l2'
        if args.is_l3_pref:
            name_prefetch_hint += '-l3'

        out_file_name = f'sample-sequential-block-{block_size}{name_prefetch_hint}.csv'
        subprocess.call(f'./bin/swpf_benchmark --gb {args.gb} --perf-profile {args.profile} --sample --sample-counter {args.sample} --out {out_file_name} --cache-lines {block_size}', shell=True)

        out_file_name = f'sample-random-block-{block_size}{name_prefetch_hint}.csv'
        subprocess.call(f'./bin/swpf_benchmark --gb {args.gb} --random --perf-profile {args.profile} --sample --sample-counter {args.sample} --out {out_file_name} --cache-lines {block_size}', shell=True)

        for prefetch_cache_lines in [1, 2, 4, 6, 8, 10, 12, 14, 16]:
            for prefetch_locality in [0, 1, 2, 3]:
                for is_interleaved in [False, True]:
                    interleaved_name = '-interleaved' if is_interleaved else ''
                    out_file_name = f'sample-locality-{prefetch_locality}-block-{block_size}-pref-{prefetch_cache_lines}{interleaved_name}{name_prefetch_hint}.csv'

                    interleaved_flag = ' --interleaved' if is_interleaved else ''
                    subprocess.call(f'./bin/swpf_benchmark --random --gb {args.gb} --perf-profile {args.profile} --sample --sample-counter {args.sample} --out {out_file_name} --cache-lines {block_size} --prefetch-lines {prefetch_cache_lines} --prefetch-distance 1 --prefetch-locality {prefetch_locality}{interleaved_flag}', shell=True)

