#!/bin/bash

# Check if the .venv directory exists
if [ ! -d ".venv" ]; then
    # The .venv directory does not exist, create it and install dependencies
    python3 -m venv .venv
    source .venv/bin/activate
    python -m pip install -r script/requirements.txt
else
    # The .venv directory exists, activate the virtual environment
    source .venv/bin/activate
fi

# Function to check for compiler existence
command_exists() {
    command -v "$1" >/dev/null 2>&1
}
if command_exists clang-17; then
    export CC_COMPILER=clang-17
    export CXX_COMPILER=clang++-17
elif command_exists clang-16; then
    export CC_COMPILER=clang-16
    export CXX_COMPILER=clang++-16
elif command_exists clang-15; then
    export CC_COMPILER=clang-15
    export CXX_COMPILER=clang++-15
elif command_exists clang-14; then
    export CC_COMPILER=clang-14
    export CXX_COMPILER=clang++-14
else
    echo "No preferred compiler found; abort."
    exit
fi

# Build
echo "Using >$CC_COMPILER< / >$CXX_COMPILER< to build the benchmark."
cmake . -DCMAKE_BUILD_TYPE=Release -DCMAKE_C_COMPILER=$CC_COMPILER -DCMAKE_CXX_COMPILER=$CXX_COMPILER
make

# Save current environment values value
original_numa_balancing=$(cat /proc/sys/kernel/numa_balancing)
echo "Current numa_balancing value: >$original_numa_balancing<"

original_governor=$(cpupower frequency-info --policy | grep 'The governor' | awk '{print $3}' | tr -d '"')
echo "Current CPU governor: >$original_governor<"

# Set desired environment values
sudo sh -c "echo 0 > /proc/sys/kernel/numa_balancing"
echo "numa_balancing set to >0<"

sudo cpupower frequency-set -g performance
echo "CPU governor set to >performance<"

# Execute trend
OUT_FILE="out.csv"
python3 script/execute.py --gb 8 --trend-only --out $OUT_FILE

# Restore original values
sudo sh -c "echo $original_numa_balancing > /proc/sys/kernel/numa_balancing"
echo "Restored numa_balancing value to >$original_numa_balancing<"

sudo cpupower frequency-set -g $original_governor
echo "Restored CPU governor to >$original_governor<"

# Bye
echo "Results are written to >$OUT_FILE<."