def generate_class(class_name: str, count_cache_lines: int, prefetched_cache_lines: list) -> str:
    code = f"""
class alignas(swprf::config::cache_line_size) {class_name}
{{
public:
    {class_name}& operator=(std::int64_t n) noexcept
    {{
        for (auto& val : value)
        {{
            val = n;
            n >>= 1;
        }}
        return *this;
    }}

    template<std::uint32_t L>
    static inline void prefetch(const {class_name}* addr)
    {{"""
    for p in prefetched_cache_lines:
        code += f"""
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + {p * 64}U), 0, L);"""
 
    code += f"""
    }}
     
    [[nodiscard]] std::uint64_t compute() const
    {{
        return {' + '.join([f'value[{c}].compute()' for c in range(0,count_cache_lines)])};
    }}
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const {class_name}* prefetch_addr) const
    {{"""
    if 0 in prefetched_cache_lines:
        code += f"""
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);
"""
    code += f"""
        auto compute = value[0].compute();
"""
    for c in range(1, count_cache_lines):
        if c in prefetched_cache_lines:
            code += f"""
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + {c * 64}U), 0, L);
"""
        code += f"""
        compute += value[{c}].compute();
"""
    code += f"""     
        return compute;
    }}
private:
    std::array<cache_line, {count_cache_lines}U> value;
}};
"""
    return code



header = '#include \"cache_line.h\"\n#include "config.h"\n'

block_sizes = [1, 2, 4, 6, 8, 10, 12, 14, 16, 24, 32]
 
## Sequential Cache lines
for cache_lines in block_sizes:
    for prefetch_lines in [1] + list(range(2, cache_lines + 2, 2)):
        if prefetch_lines <= cache_lines:
            header += generate_class(
                f'cache_lines_{cache_lines}_pref_{prefetch_lines}_seq',
                cache_lines,
                list(range(0, prefetch_lines))
            )
        
## Adjacent Cache lines (skip single cache line)
for cache_lines in block_sizes[1:]:
    header += generate_class(
        f'cache_lines_{cache_lines}_pref_{int(cache_lines / 2)}_adj', 
        cache_lines, 
        list(range(0, cache_lines, 2))
    )

with open('cache_lines.h', 'w') as out:
    out.write(header)
