#pragma once

#include <array>
#include <cstdint>
#include "config.h"
#include "compute.h"

class alignas(swprf::config::cache_line_size) cache_line
{
public:
    std::array<std::int32_t, 16U> value;

    cache_line& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    [[nodiscard]] std::uint64_t light_compute() const
    {
        return value[0] + value[1] + value[2] + value[3]
            + value[4] + value[5] + value[6] + value[7]
            + value[8] + value[9] + value[10] + value[11]
            + value[12] + value[13] + value[14] + value[15];
    }

    [[nodiscard]] std::uint64_t compute() const
    {
        return swprf::Compute::work(value[1])
               + swprf::Compute::work(value[2])
               + swprf::Compute::work(value[3])
               + swprf::Compute::work(value[4])
               + swprf::Compute::work(value[5])
               + swprf::Compute::work(value[6])
               + swprf::Compute::work(value[7])
               + swprf::Compute::work(value[8])
               + swprf::Compute::work(value[9])
               + swprf::Compute::work(value[10])
               + swprf::Compute::work(value[11])
               + swprf::Compute::work(value[12])
               + swprf::Compute::work(value[13])
               //+ swprf::Compute::work(value[14])
               //+ swprf::Compute::work(value[15])
                ;
    }

    [[nodiscard]] std::uint64_t heavy_compute() const
    {
        return swprf::Compute::heavy_work(value[0])
               + swprf::Compute::heavy_work(value[1])
               + swprf::Compute::heavy_work(value[2])
               + swprf::Compute::heavy_work(value[3])
               + swprf::Compute::heavy_work(value[4])
               + swprf::Compute::heavy_work(value[5])
               + swprf::Compute::heavy_work(value[6])
               + swprf::Compute::heavy_work(value[7])
               + swprf::Compute::heavy_work(value[8])
               + swprf::Compute::heavy_work(value[9])
               + swprf::Compute::heavy_work(value[10])
               + swprf::Compute::heavy_work(value[11])
               + swprf::Compute::heavy_work(value[12])
               + swprf::Compute::heavy_work(value[13])
               + swprf::Compute::heavy_work(value[14])
               + swprf::Compute::heavy_work(value[15])
                ;
    }
};
