#pragma once
#include <chrono>
#include <cstdint>
#include <iostream>
#include <pthread.h>
#include <string>
#include <thread>

namespace swprf {
/**
 * Encapsulates methods for thread access.
 */
class thread
{
public:
    /**
     * Pins a thread to a given core.
     *
     * @param thread Thread to pin.
     * @param core_id Core where the thread should be pinned.
     * @return True, when pinning was successful.
     */
    static bool pin(std::thread &thread, const std::uint16_t core_id)
    {
        cpu_set_t cpu_set;
        CPU_ZERO(&cpu_set);
        CPU_SET(core_id, &cpu_set);

        if (pthread_setaffinity_np(thread.native_handle(), sizeof(cpu_set_t), &cpu_set) != 0)
        {
            std::cerr << "Can not pin thread!" << std::endl;
            return false;
        }

        return true;
    }

    static void name(std::thread &thread, std::string &&name)
    {
        pthread_setname_np(thread.native_handle(), name.c_str());
    }
};
} // namespace mx::system