#include "cache_line.h"
#include "config.h"

class alignas(swprf::config::cache_line_size) cache_lines_1_pref_1_seq
{
public:
    cache_lines_1_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_1_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_1_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 1U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_2_pref_1_seq
{
public:
    cache_lines_2_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_2_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_2_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 2U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_2_pref_2_seq
{
public:
    cache_lines_2_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_2_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_2_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 2U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_4_pref_1_seq
{
public:
    cache_lines_4_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_4_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_4_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 4U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_4_pref_2_seq
{
public:
    cache_lines_4_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_4_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_4_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 4U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_4_pref_4_seq
{
public:
    cache_lines_4_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_4_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_4_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 4U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_6_pref_1_seq
{
public:
    cache_lines_6_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_6_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_6_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 6U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_6_pref_2_seq
{
public:
    cache_lines_6_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_6_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_6_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 6U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_6_pref_4_seq
{
public:
    cache_lines_6_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_6_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_6_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 6U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_6_pref_6_seq
{
public:
    cache_lines_6_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_6_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_6_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 6U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_1_seq
{
public:
    cache_lines_8_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_2_seq
{
public:
    cache_lines_8_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_4_seq
{
public:
    cache_lines_8_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_6_seq
{
public:
    cache_lines_8_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_8_seq
{
public:
    cache_lines_8_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_1_seq
{
public:
    cache_lines_10_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_2_seq
{
public:
    cache_lines_10_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_4_seq
{
public:
    cache_lines_10_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_6_seq
{
public:
    cache_lines_10_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_8_seq
{
public:
    cache_lines_10_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_10_seq
{
public:
    cache_lines_10_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_1_seq
{
public:
    cache_lines_12_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_2_seq
{
public:
    cache_lines_12_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_4_seq
{
public:
    cache_lines_12_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_6_seq
{
public:
    cache_lines_12_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_8_seq
{
public:
    cache_lines_12_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_10_seq
{
public:
    cache_lines_12_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_12_seq
{
public:
    cache_lines_12_pref_12_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_12_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_12_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_1_seq
{
public:
    cache_lines_14_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_2_seq
{
public:
    cache_lines_14_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_4_seq
{
public:
    cache_lines_14_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_6_seq
{
public:
    cache_lines_14_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_8_seq
{
public:
    cache_lines_14_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_10_seq
{
public:
    cache_lines_14_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_12_seq
{
public:
    cache_lines_14_pref_12_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_12_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_12_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_14_seq
{
public:
    cache_lines_14_pref_14_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_14_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_14_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_1_seq
{
public:
    cache_lines_16_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_2_seq
{
public:
    cache_lines_16_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_4_seq
{
public:
    cache_lines_16_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_6_seq
{
public:
    cache_lines_16_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_8_seq
{
public:
    cache_lines_16_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_10_seq
{
public:
    cache_lines_16_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_12_seq
{
public:
    cache_lines_16_pref_12_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_12_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_12_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_14_seq
{
public:
    cache_lines_16_pref_14_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_14_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_14_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_16_seq
{
public:
    cache_lines_16_pref_16_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_16_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_16_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_1_seq
{
public:
    cache_lines_24_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_2_seq
{
public:
    cache_lines_24_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_4_seq
{
public:
    cache_lines_24_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_6_seq
{
public:
    cache_lines_24_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_8_seq
{
public:
    cache_lines_24_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_10_seq
{
public:
    cache_lines_24_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_12_seq
{
public:
    cache_lines_24_pref_12_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_12_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_12_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_14_seq
{
public:
    cache_lines_24_pref_14_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_14_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_14_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_16_seq
{
public:
    cache_lines_24_pref_16_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_16_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_16_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_18_seq
{
public:
    cache_lines_24_pref_18_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_18_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_18_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_20_seq
{
public:
    cache_lines_24_pref_20_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_20_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_20_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_22_seq
{
public:
    cache_lines_24_pref_22_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_22_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_22_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_24_seq
{
public:
    cache_lines_24_pref_24_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_24_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_24_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_1_seq
{
public:
    cache_lines_32_pref_1_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_1_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_1_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_2_seq
{
public:
    cache_lines_32_pref_2_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_2_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_2_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        compute += value[2].compute();

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_4_seq
{
public:
    cache_lines_32_pref_4_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_4_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_4_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        compute += value[4].compute();

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_6_seq
{
public:
    cache_lines_32_pref_6_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_6_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_6_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        compute += value[6].compute();

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_8_seq
{
public:
    cache_lines_32_pref_8_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_8_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_8_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        compute += value[8].compute();

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_10_seq
{
public:
    cache_lines_32_pref_10_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_10_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_10_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        compute += value[10].compute();

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_12_seq
{
public:
    cache_lines_32_pref_12_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_12_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_12_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        compute += value[12].compute();

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_14_seq
{
public:
    cache_lines_32_pref_14_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_14_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_14_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        compute += value[14].compute();

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_16_seq
{
public:
    cache_lines_32_pref_16_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_16_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_16_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        compute += value[16].compute();

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_18_seq
{
public:
    cache_lines_32_pref_18_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_18_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_18_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        compute += value[18].compute();

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_20_seq
{
public:
    cache_lines_32_pref_20_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_20_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_20_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        compute += value[20].compute();

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_22_seq
{
public:
    cache_lines_32_pref_22_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_22_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_22_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        compute += value[22].compute();

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_24_seq
{
public:
    cache_lines_32_pref_24_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_24_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_24_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();

        compute += value[24].compute();

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_26_seq
{
public:
    cache_lines_32_pref_26_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_26_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1536U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1600U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_26_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1536U), 0, L);

        compute += value[24].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1600U), 0, L);

        compute += value[25].compute();

        compute += value[26].compute();

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_28_seq
{
public:
    cache_lines_32_pref_28_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_28_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1536U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1600U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1664U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1728U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_28_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1536U), 0, L);

        compute += value[24].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1600U), 0, L);

        compute += value[25].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1664U), 0, L);

        compute += value[26].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1728U), 0, L);

        compute += value[27].compute();

        compute += value[28].compute();

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_30_seq
{
public:
    cache_lines_32_pref_30_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_30_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1536U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1600U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1664U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1728U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1792U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1856U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_30_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1536U), 0, L);

        compute += value[24].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1600U), 0, L);

        compute += value[25].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1664U), 0, L);

        compute += value[26].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1728U), 0, L);

        compute += value[27].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1792U), 0, L);

        compute += value[28].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1856U), 0, L);

        compute += value[29].compute();

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_32_seq
{
public:
    cache_lines_32_pref_32_seq& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_32_seq* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 64U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 192U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 320U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 448U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 576U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 704U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 832U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 960U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1088U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1216U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1344U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1472U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1536U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1600U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1664U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1728U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1792U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1856U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1920U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1984U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_32_seq* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 64U), 0, L);

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 192U), 0, L);

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 320U), 0, L);

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 448U), 0, L);

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 576U), 0, L);

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 704U), 0, L);

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 832U), 0, L);

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 960U), 0, L);

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1088U), 0, L);

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1216U), 0, L);

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1344U), 0, L);

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1472U), 0, L);

        compute += value[23].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1536U), 0, L);

        compute += value[24].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1600U), 0, L);

        compute += value[25].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1664U), 0, L);

        compute += value[26].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1728U), 0, L);

        compute += value[27].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1792U), 0, L);

        compute += value[28].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1856U), 0, L);

        compute += value[29].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1920U), 0, L);

        compute += value[30].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1984U), 0, L);

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_2_pref_1_adj
{
public:
    cache_lines_2_pref_1_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_2_pref_1_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_2_pref_1_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 2U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_4_pref_2_adj
{
public:
    cache_lines_4_pref_2_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_4_pref_2_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_4_pref_2_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 4U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_6_pref_3_adj
{
public:
    cache_lines_6_pref_3_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_6_pref_3_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_6_pref_3_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 6U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_8_pref_4_adj
{
public:
    cache_lines_8_pref_4_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_8_pref_4_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_8_pref_4_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 8U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_10_pref_5_adj
{
public:
    cache_lines_10_pref_5_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_10_pref_5_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_10_pref_5_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 10U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_12_pref_6_adj
{
public:
    cache_lines_12_pref_6_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_12_pref_6_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_12_pref_6_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        compute += value[11].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 12U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_14_pref_7_adj
{
public:
    cache_lines_14_pref_7_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_14_pref_7_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_14_pref_7_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        compute += value[13].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 14U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_16_pref_8_adj
{
public:
    cache_lines_16_pref_8_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_16_pref_8_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_16_pref_8_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        compute += value[15].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 16U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_24_pref_12_adj
{
public:
    cache_lines_24_pref_12_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_24_pref_12_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_24_pref_12_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        compute += value[23].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 24U> value;
};

class alignas(swprf::config::cache_line_size) cache_lines_32_pref_16_adj
{
public:
    cache_lines_32_pref_16_adj& operator=(std::int64_t n) noexcept
    {
        for (auto& val : value)
        {
            val = n;
            n >>= 1;
        }
        return *this;
    }

    template<std::uint32_t L>
    static inline void prefetch(const cache_lines_32_pref_16_adj* addr)
    {
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 0U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 128U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 256U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 384U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 512U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 640U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 768U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 896U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1024U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1152U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1280U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1408U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1536U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1664U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1792U), 0, L);
        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(addr) + 1920U), 0, L);
    }
     
    [[nodiscard]] std::uint64_t compute() const
    {
        return value[0].compute() + value[1].compute() + value[2].compute() + value[3].compute() + value[4].compute() + value[5].compute() + value[6].compute() + value[7].compute() + value[8].compute() + value[9].compute() + value[10].compute() + value[11].compute() + value[12].compute() + value[13].compute() + value[14].compute() + value[15].compute() + value[16].compute() + value[17].compute() + value[18].compute() + value[19].compute() + value[20].compute() + value[21].compute() + value[22].compute() + value[23].compute() + value[24].compute() + value[25].compute() + value[26].compute() + value[27].compute() + value[28].compute() + value[29].compute() + value[30].compute() + value[31].compute();
    }
    
    template<std::uint32_t L>
    [[nodiscard]] std::uint64_t compute_and_prefetch_interleaved(const cache_lines_32_pref_16_adj* prefetch_addr) const
    {
         __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr)), 0, L);

        auto compute = value[0].compute();

        compute += value[1].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 128U), 0, L);

        compute += value[2].compute();

        compute += value[3].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 256U), 0, L);

        compute += value[4].compute();

        compute += value[5].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 384U), 0, L);

        compute += value[6].compute();

        compute += value[7].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 512U), 0, L);

        compute += value[8].compute();

        compute += value[9].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 640U), 0, L);

        compute += value[10].compute();

        compute += value[11].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 768U), 0, L);

        compute += value[12].compute();

        compute += value[13].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 896U), 0, L);

        compute += value[14].compute();

        compute += value[15].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1024U), 0, L);

        compute += value[16].compute();

        compute += value[17].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1152U), 0, L);

        compute += value[18].compute();

        compute += value[19].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1280U), 0, L);

        compute += value[20].compute();

        compute += value[21].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1408U), 0, L);

        compute += value[22].compute();

        compute += value[23].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1536U), 0, L);

        compute += value[24].compute();

        compute += value[25].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1664U), 0, L);

        compute += value[26].compute();

        compute += value[27].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1792U), 0, L);

        compute += value[28].compute();

        compute += value[29].compute();

        __builtin_prefetch(reinterpret_cast<void*>(std::uintptr_t(prefetch_addr) + 1920U), 0, L);

        compute += value[30].compute();

        compute += value[31].compute();
     
        return compute;
    }
private:
    std::array<cache_line, 32U> value;
};
